# MortalityTables

[![Stable](https://img.shields.io/badge/docs-stable-blue.svg)](https://alecloudenback.github.io/MortalityTables.jl/stable)
[![Dev](https://img.shields.io/badge/docs-dev-blue.svg)](https://alecloudenback.github.io/MortalityTables.jl/dev)
[![Build Status](https://travis-ci.com/JuliaActuary/MortalityTables.jl.svg?branch=master)](https://travis-ci.com/JuliaActuary/MortalityTables.jl)
[![codecov](https://codecov.io/gh/JuliaActuary/MortalityTables.jl/branch/master/graph/badge.svg)](https://codecov.io/gh/JuliaActuary/MortalityTables.jl)

A Julia package for working with MortalityTables. Has first-class support for missing values.

## Examples
### Quickstart

```julia
using MortalityTables

tables = MortalityTables.tables() # loads the tables stored in the package folder
vbt2001 = tables["2001 VBT Residual Standard Select and Ultimate - Male Nonsmoker, ANB"]

# indexed by issue age and duration for select rate
q(vbt2001.select,35,1)        # .00036
q(vbt2001.ultimate,95,1)          # .24298

# when accessing ultimate rates, don't always need to specify the duration
q(vbt2001.ultimate,95)          # .24298
```

### Example: Quickly access and compare tables
```julia
using MortalityTables, Plots


tables = MortalityTables.tables()
cso_2001 = tables["2001 CSO Super Preferred Select and Ultimate - Male Nonsmoker, ANB"]
cso_1980 = tables["1980 CSO - Male Nonsmoker, ANB"]

issue_age = 27
durations = 1:30
mort = [q(cso_2001.select,issue_age,durations),
        q(cso_1980.ultimate,issue_age,durations)]
plot(
   durations,
   mort,
   label = ["2001 CSO M SuperPref NS" "1980 CSO M NS"],
   title = "Comparison of 1980 and 2001 CSO \n for 27-year-old male",
   xlabel="duration")
```
![Comparison of 2001 and 1980 CSO](https://user-images.githubusercontent.com/711879/76151403-739a7380-607a-11ea-861b-8fe9fe6607c4.png)


## Usage

### Indexing

Tables are indexed by a starting age and duration (even ultimate tables, under the hood). For tables with a starting age that is defined, but you've requested an attained age beyond the defined rates, you will get a `BoundsError`. If you ask for a starting age that is not defined, you will get a `missing`.

The rationale for this is, for example, this [2001 CSO table](https://mort.soa.org/ViewTable.aspx?&TableIdentity=1076) is not defined for ages 15 and under and ends at 120.
- You will get a `missing` if you ask for starting age 10 or 15, because it's plausible that you could encounter a a starting age not defined by a table.
- You will get a `BoundsError` if you ask for an attained age 150 for someone select at age 16, because it is beyond the table's definition of its end.

#### Index by issue age and duration

```julia
q(vbt2001.select,35,1)        # .00036
q(vbt2001.select,35,61)       # .24298

# can easily get ranges of values:
q(vbt2001.select,35,1:30)     # [0.0036, 0.0048, ...]
```

#### Index by just age to get the ultimate rates
```julia
q(vbt2001,95)          # .24298
q(vbt2001,50:70)       # [0.00319, 0.00345, ...]
```

### Other Usage

#### Table Attributes
```julia
issue_age = 50
ω(vbt2001.select,issue_age)              # 120
omega(vbt2001.ultimate, issue_age)          # 120
```

#### Table MetaData

When you have an expression that shows a Mortality table, it displays relevant information:

```julia
tables = MortalityTables.tables()
vbt2001 = tables["2001 VBT Residual Standard Select and Ultimate - Male Nonsmoker, ANB"]
```

This shows the following in a notebook or REPL:

```
MortalityTable:
   Name:
       2001 VBT Residual Standard Select and Ultimate - Male Nonsmoker, ANB
   Provider:
       Society of Actuaries
   mort.SOA.org ID:
       1118
   mort.SOA.org link:
       https://mort.soa.org/ViewTable.aspx?&TableIdentity=1118
   Description:
       2001 Valuation Basic Table (VBT) Residual Standard Select and Ultimate Table -  Male Nonsmoker. Basis: Age Nearest Birthday. Minimum Select Age: 0. Maximum Select Age: 99. Minimum Ultimate Age: 25. Maximum Ultimate Age: 120
```

#### Fractional Years
When evaluating survival over partial years when you are given full year mortality
rates, you must make an assumption over how those deaths are distributed throughout
the year. Three assumptions are provided as options and are based on formulas
from the [2016 Experience Study Calculations paper from the SOA](https://www.soa.org/globalassets/assets/Files/Research/2016-10-experience-study-calculations.pdf), specifically pages 40-44.

The three assumptions are:

- `Uniform()` which assumes an increasing force of mortality throughout the year.
- `Constant()` which assumes a level force of mortality throughout the year.
- `Balducci()` which assumes a decreasing force of mortality over the year. It seems [to
be for](https://www.soa.org/globalassets/assets/library/research/actuarial-research-clearing-house/1978-89/1988/arch-1/arch88v17.pdf) making it easier to calculate successive months by hand.

##### Usage

When you call a method below that uses the `time` argument (ie a period over which
    you want to calculate the force of mortality), if you use a non `Int` (integer)
    number then you need to specify the assumption as the last argument.

For example:

**_Don't_** need to specify because you gave an `Int` time:

```julia
# calculate the 5-year survival for a person issued at age 50 and in
# the first duration
issue_age = 50
duration = 1
time = 5
p(table,issue_age,duration,time)
```
**_Do_** need to specify because you gave an fractional floating time:

```julia
# calculate the 5-and-a-half-year survival for a
# person issued at age 50 and in the first duration
issue_age = 50
duration = 1
time = 5.5
p(table,issue_age,duration,time, Balducci())
```

Note that if you are passing floating point numbers as the `time`
argument, you still have to specify a mortality assumption because
[floating point numbers](https://en.wikipedia.org/wiki/Floating-point_arithmetic) are often *technically* not whole numbers even
if you define a number to be, say, `5.0`.


### Actuarial Notation Equivalants
#### `ₜp₍ₓ₎₊ₛ`
The probability that a life aged `x + s` who was select
at age `x` survives to at least age `x+s+t`.

```julia
issue_age = x
duration = s - 1
time = t
p(table,issue_age,duration,time)
```

#### `ₜpₓ`
The probability that a life aged `x` survives to at least age `t`.

```julia
issue_age = x
duration = 1
time = t
p(table,issue_age,duration,time)
```

#### `pₓ`
The probability that a life aged `x` survives through age `x+1`

```julia
issue_age = x
duration = 1
time = 1
p(table,issue_age,duration,time)
```

#### `ₜqₓ`
The probability that a life aged `x` dies by age `x+t`

```julia
issue_age = x
duration = 1
time = t
q(table,issue_age,duration,time)
```

#### `qₓ`
The probability that a life aged `x` dies by age `x+1`

```julia
issue_age = x
duration = 1
time = 1
q(table,issue_age,duration,time)
```

#### `ω`
Returns the last attained age which the table has defined (ie not including) for a given issue_age.

```julia
omega(table, issue_age)
ω(table, issue_age)
```

### Some Batteries Included

Comes with some tables built in via [mort.SOA.org](https://mort.soa.org) and by using [you agree to their terms](https://mort.soa.org/TermsOfUse.aspx).

Not all tables have been tested that they work by default, though I have not encountered issues with any of the the VBT/CSO/other usual tables.

Sample of some of the included table sets:
```
2017 Loaded CSO
2015 VBT
2001 VBT
2001 CSO
1980 CSO
1980 CET
```

[Click here to see the full list of tables included.](BundledTables.md)



### Adding more tables

To add more tables for your use, download the `.xml` (aka the (`Xtbml` format)[https://mort.soa.org/About.aspx]) version of the table from [mort.SOA.org](https://mort.soa.org) and place it in the directory the package is installed in. This is usually `~user/.julia/packages/MortalityTables/[changing hash value]/src/tables/`. *Note: updating the package may remove your existing tables. Make a backup before updating your packages*

After placing packages in the folder above, restart Julia and the should be discoverable when you run `mt.Tables()`

If you would like more tables added by default, please open a GitHub issue with the request.


### References
- [SOA Mortality Tables](https://mort.soa.org/)
- [Actuarial Mathematics for Life Contingent Risks, 2nd ed](https://www.cambridge.org/vi/academic/subjects/statistics-probability/statistics-econometrics-finance-and-insurance/actuarial-mathematics-life-contingent-risks-2nd-edition?format=HB)
- [Experience Study Calculations, SOA](https://www.soa.org/globalassets/assets/Files/Research/2016-10-experience-study-calculations.pdf)

### Similar Projects
 - [Pyliferisk, a Python package](https://github.com/franciscogarate/pyliferisk)
